@@ -9,6 +9,7 @@
 
 #include <aws/auth/credentials.h>
 #include <aws/auth/private/credentials_utils.h>
+#include <aws/common/byte_buf.h>
 #include <aws/common/clock.h>
 #include <aws/common/condition_variable.h>
 #include <aws/common/date_time.h>
@@ -16,6 +17,7 @@
 #include <aws/common/string.h>
 #include <aws/common/thread.h>
 #include <aws/common/uri.h>
+#include <aws/http/proxy.h>
 #include <aws/http/request_response.h>
 #include <aws/http/status_code.h>
 #include <aws/io/channel_bootstrap.h>
@@ -35,6 +37,7 @@ struct aws_mock_ecs_tester {
     struct aws_string *request_path_and_query;
     struct aws_string *request_authorization_header;
     struct aws_string *selected_host;
+    int attempts;
 
     struct aws_array_list response_data_callbacks;
     bool is_connection_acquire_successful;
@@ -55,6 +58,8 @@ struct aws_mock_ecs_tester {
     struct aws_host_resolver *host_resolver;
     struct aws_client_bootstrap *bootstrap;
     struct aws_tls_ctx *tls_ctx;
+
+    struct proxy_env_var_settings *proxy_config;
 };
 
 static struct aws_mock_ecs_tester s_tester;
@@ -94,6 +99,12 @@ static struct aws_http_connection_manager *s_aws_http_connection_manager_new_moc
     s_tester.selected_tls = options->tls_connection_options != NULL;
     aws_mutex_unlock(&s_tester.lock);
 
+    if (s_tester.proxy_config != NULL) {
+        AWS_FATAL_ASSERT(options->proxy_ev_settings->env_var_type == s_tester.proxy_config->env_var_type);
+        AWS_FATAL_ASSERT(options->proxy_ev_settings->connection_type == s_tester.proxy_config->connection_type);
+        AWS_FATAL_ASSERT(options->proxy_ev_settings->tls_options == s_tester.proxy_config->tls_options);
+    }
+
     return (struct aws_http_connection_manager *)1;
 }
 
@@ -173,10 +184,14 @@ static struct aws_http_stream *s_aws_http_connection_make_request_mock(
     (void)client_connection;
     (void)options;
 
+    s_tester.attempts++;
     struct aws_byte_cursor path;
     AWS_ZERO_STRUCT(path);
     aws_http_message_get_request_path(options->request, &path);
-
+    if (s_tester.request_path_and_query != NULL) {
+        aws_string_destroy(s_tester.request_path_and_query);
+        s_tester.request_path_and_query = NULL;
+    }
     s_tester.request_path_and_query = aws_string_new_from_cursor(s_tester.allocator, &path);
     struct aws_byte_cursor authorization_header_value;
     AWS_ZERO_STRUCT(authorization_header_value);
@@ -184,6 +199,10 @@ static struct aws_http_stream *s_aws_http_connection_make_request_mock(
             aws_http_message_get_headers(options->request),
             aws_byte_cursor_from_c_str("Authorization"),
             &authorization_header_value) == AWS_OP_SUCCESS) {
+        if (s_tester.request_authorization_header != NULL) {
+            aws_string_destroy(s_tester.request_authorization_header);
+            s_tester.request_authorization_header = NULL;
+        }
         s_tester.request_authorization_header =
             aws_string_new_from_cursor(s_tester.allocator, &authorization_header_value);
     }
@@ -431,6 +450,7 @@ static int s_credentials_provider_ecs_request_failure(struct aws_allocator *allo
     ASSERT_TRUE(s_tester.has_received_credentials_callback == true);
     ASSERT_TRUE(s_tester.credentials == NULL);
     ASSERT_UINT_EQUALS(443, s_tester.selected_port);
+    ASSERT_UINT_EQUALS(4, s_tester.attempts);
     aws_mutex_unlock(&s_tester.lock);
 
     aws_credentials_provider_release(provider);
@@ -546,9 +566,14 @@ AWS_STATIC_STRING_FROM_LITERAL(
     s_good_response,
     "{\"AccessKeyId\":\"SuccessfulAccessKey\", \n  \"SecretAccessKey\":\"SuccessfulSecret\", \n  "
     "\"Token\":\"TokenSuccess\", \n \"Expiration\":\"2020-02-25T06:03:31Z\"}");
+AWS_STATIC_STRING_FROM_LITERAL(
+    s_good_response_with_account_id,
+    "{\"AccessKeyId\":\"SuccessfulAccessKey\", \n  \"SecretAccessKey\":\"SuccessfulSecret\", \n  "
+    "\"Token\":\"TokenSuccess\", \n \"Expiration\":\"2020-02-25T06:03:31Z\", \"AccountId\":\"AccountId123\"}");
 AWS_STATIC_STRING_FROM_LITERAL(s_good_access_key_id, "SuccessfulAccessKey");
 AWS_STATIC_STRING_FROM_LITERAL(s_good_secret_access_key, "SuccessfulSecret");
 AWS_STATIC_STRING_FROM_LITERAL(s_good_session_token, "TokenSuccess");
+AWS_STATIC_STRING_FROM_LITERAL(s_good_account_id, "AccountId123");
 AWS_STATIC_STRING_FROM_LITERAL(s_good_response_expiration, "2020-02-25T06:03:31Z");
 
 /* Check that expected URI and Authorization token were used to make request.
@@ -580,7 +605,8 @@ static int s_do_ecs_success_test(
     struct aws_allocator *allocator,
     struct aws_credentials_provider_ecs_options *options,
     const char *expected_uri,
-    const char *expected_token) {
+    const char *expected_token,
+    const struct aws_string *expected_account_id) {
     struct aws_credentials_provider *provider = aws_credentials_provider_new_ecs(allocator, options);
 
     aws_credentials_provider_get_credentials(provider, s_get_credentials_callback, NULL);
@@ -595,6 +621,11 @@ static int s_do_ecs_success_test(
     ASSERT_CURSOR_VALUE_STRING_EQUALS(
         aws_credentials_get_secret_access_key(s_tester.credentials), s_good_secret_access_key);
     ASSERT_CURSOR_VALUE_STRING_EQUALS(aws_credentials_get_session_token(s_tester.credentials), s_good_session_token);
+    if (expected_account_id) {
+        ASSERT_CURSOR_VALUE_STRING_EQUALS(aws_credentials_get_account_id(s_tester.credentials), expected_account_id);
+    } else {
+        ASSERT_TRUE(aws_credentials_get_account_id(s_tester.credentials).len == 0);
+    }
 
     struct aws_date_time expiration;
     struct aws_byte_cursor date_cursor = aws_byte_cursor_from_string(s_good_response_expiration);
@@ -716,7 +747,8 @@ static int s_credentials_provider_ecs_basic_success(struct aws_allocator *alloca
         allocator,
         &options,
         "https://www.xxx123321testmocknonexsitingawsservice.com:443/path/to/resource/?a=b&c=d" /*expected_uri*/,
-        "test-token-1234-abcd" /*expected_token*/));
+        "test-token-1234-abcd" /*expected_token*/,
+        NULL /*expected_account_id*/));
 
     s_aws_ecs_tester_cleanup();
 
@@ -725,6 +757,44 @@ static int s_credentials_provider_ecs_basic_success(struct aws_allocator *alloca
 
 AWS_TEST_CASE(credentials_provider_ecs_basic_success, s_credentials_provider_ecs_basic_success);
 
+static int s_credentials_provider_ecs_basic_success_with_account_id(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+
+    s_aws_ecs_tester_init(allocator);
+
+    struct aws_byte_cursor good_response_cursor = aws_byte_cursor_from_string(s_good_response_with_account_id);
+    aws_array_list_push_back(&s_tester.response_data_callbacks, &good_response_cursor);
+
+    struct aws_credentials_provider_ecs_options options = {
+        .bootstrap = s_tester.bootstrap,
+        .function_table = &s_mock_function_table,
+        .shutdown_options =
+            {
+                .shutdown_callback = s_on_shutdown_complete,
+                .shutdown_user_data = NULL,
+            },
+        .host = aws_byte_cursor_from_c_str("www.xxx123321testmocknonexsitingawsservice.com"),
+        .path_and_query = aws_byte_cursor_from_c_str("/path/to/resource/?a=b&c=d"),
+        .auth_token = aws_byte_cursor_from_c_str("test-token-1234-abcd"),
+        .tls_ctx = s_tester.tls_ctx,
+    };
+
+    ASSERT_SUCCESS(s_do_ecs_success_test(
+        allocator,
+        &options,
+        "https://www.xxx123321testmocknonexsitingawsservice.com:443/path/to/resource/?a=b&c=d" /*expected_uri*/,
+        "test-token-1234-abcd" /*expected_token*/,
+        s_good_account_id /*expected_account_id*/));
+
+    s_aws_ecs_tester_cleanup();
+
+    return 0;
+}
+
+AWS_TEST_CASE(
+    credentials_provider_ecs_basic_success_with_account_id,
+    s_credentials_provider_ecs_basic_success_with_account_id);
+
 static int s_credentials_provider_ecs_basic_success_token_file(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
 
@@ -766,8 +836,7 @@ static int s_credentials_provider_ecs_basic_success_token_file(struct aws_alloca
         "https://www.xxx123321testmocknonexsitingawsservice.com:443/path/to/resource/?a=b&c=d",
         aws_string_c_str(auth_token)));
 
-    aws_string_destroy(s_tester.request_path_and_query);
-    aws_string_destroy(s_tester.request_authorization_header);
+    s_tester.has_received_credentials_callback = false;
     aws_credentials_release(s_tester.credentials);
 
     aws_mutex_unlock(&s_tester.lock);
@@ -973,7 +1042,8 @@ static int s_credentials_provider_ecs_no_auth_token_success(struct aws_allocator
         allocator,
         &options,
         "https://www.xxx123321testmocknonexsitingawsservice.com:443/path/to/resource/?a=b&c=d" /*expected_uri*/,
-        NULL /*expected_token*/));
+        NULL /*expected_token*/,
+        NULL /*expected_account_id=*/));
 
     s_aws_ecs_tester_cleanup();
 
@@ -1163,3 +1233,50 @@ static int s_credentials_provider_ecs_real_success(struct aws_allocator *allocat
 }
 
 AWS_TEST_CASE(credentials_provider_ecs_real_success, s_credentials_provider_ecs_real_success);
+
+static int s_credentials_provider_ecs_proxy_routing_enabled_test(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+
+    s_aws_ecs_tester_init(allocator);
+
+    struct proxy_env_var_settings proxy_config = {
+        .env_var_type = AWS_HPEV_ENABLE,
+    };
+
+    s_tester.proxy_config = &proxy_config;
+
+    struct aws_credentials_provider_ecs_options options = {
+        .bootstrap = s_tester.bootstrap,
+        .function_table = &s_mock_function_table,
+        .shutdown_options =
+            {
+                .shutdown_callback = s_on_shutdown_complete,
+                .shutdown_user_data = NULL,
+            },
+        .host = aws_byte_cursor_from_c_str("127.0.0.1"),
+        .path_and_query = aws_byte_cursor_from_c_str("/path"),
+        .tls_ctx = s_tester.tls_ctx,
+        .proxy_ev_settings = &proxy_config,
+    };
+
+    struct aws_credentials_provider *provider = aws_credentials_provider_new_ecs(allocator, &options);
+
+    aws_credentials_provider_get_credentials(provider, s_get_credentials_callback, NULL);
+
+    s_aws_wait_for_credentials_result();
+
+    ASSERT_TRUE(s_tester.credentials == NULL);
+
+    aws_credentials_provider_release(provider);
+
+    s_aws_wait_for_provider_shutdown_callback();
+
+    aws_mem_release(provider->allocator, provider);
+
+    s_aws_ecs_tester_cleanup();
+
+    return 0;
+}
+AWS_TEST_CASE(
+    credentials_provider_ecs_proxy_routing_enabled_test,
+    s_credentials_provider_ecs_proxy_routing_enabled_test);
